@@ -327,6 +327,12 @@ int get_curve_name(int key_algorithm) {
     return NID_X9_62_prime256v1;
   } else if(key_algorithm == YKPIV_ALGO_ECCP384) {
     return NID_secp384r1;
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+  } else if(key_algorithm == YKPIV_ALGO_ED25519) {
+    return NID_ED25519;
+  } else if(key_algorithm == YKPIV_ALGO_X25519) {
+    return NID_X25519;
+#endif
   }
   return 0;
 }
@@ -376,8 +382,10 @@ int get_slot_hex(enum enum_slot slot_enum) {
   return slot;
 }
 
-bool set_component(unsigned char *in_ptr, const BIGNUM *bn, int element_len) {
-  return BN_bn2binpad(bn, in_ptr, element_len) == element_len;
+bool set_component(unsigned char *in_ptr, const BIGNUM *bn, int *element_len) {
+  if(BN_num_bytes(bn) > *element_len) return false;
+  *element_len = BN_bn2bin(bn, in_ptr);
+  return true;
 }
 
 bool prepare_rsa_signature(const unsigned char *in, unsigned int in_len, unsigned char *out, unsigned int *out_len, int nid) {
@@ -569,6 +577,10 @@ unsigned char get_pin_policy(enum enum_pin_policy policy) {
       return YKPIV_PINPOLICY_ONCE;
     case pin_policy_arg_always:
       return YKPIV_PINPOLICY_ALWAYS;
+    case pin_policy_arg_matchonce:
+      return YKPIV_PINPOLICY_MATCH_ONCE;
+    case pin_policy_arg_matchalways:
+      return YKPIV_PINPOLICY_MATCH_ALWAYS;
     case pin_policy__NULL:
     default:
       return YKPIV_PINPOLICY_DEFAULT;
@@ -602,26 +614,27 @@ int SSH_write_X509(FILE *fp, X509 *x) {
 
   switch (EVP_PKEY_base_id(pkey)) {
   case EVP_PKEY_RSA: {
-    RSA *rsa;
+    const RSA *rsa;
     unsigned char n[256] = {0};
     const BIGNUM *bn_n;
 
     char rsa_id[] = "\x00\x00\x00\x07ssh-rsa";
     char rsa_f4[] = "\x00\x00\x00\x03\x01\x00\x01";
 
-    rsa = EVP_PKEY_get1_RSA(pkey);
+    rsa = EVP_PKEY_get0_RSA(pkey);
     if(rsa == NULL) {
       break;
     }
     RSA_get0_key(rsa, &bn_n, NULL, NULL);
 
-    if (!set_component(n, bn_n, RSA_size(rsa))) {
+    int len = RSA_size(rsa);
+    if (!set_component(n, bn_n, &len)) {
       break;
     }
 
-    uint32_t bytes = BN_num_bytes(bn_n);
+    uint32_t bytes = len;
     char len_buf[5] = {0};
-    int len = 4;
+    len = 4;
 
     len_buf[0] = (bytes >> 24) & 0x000000ff;
     len_buf[1] = (bytes << 16) & 0x000000ff;
@@ -658,7 +671,7 @@ int SSH_write_X509(FILE *fp, X509 *x) {
       BIO_free_all(b64);
       break;
     }
-    if(BIO_write(b64, n, RSA_size(rsa)) <= 0) {
+    if(BIO_write(b64, n, bytes) <= 0) {
       fprintf(stderr, "Failed to write RSA n component\n");
       BIO_free_all(b64);
       break;
@@ -679,3 +692,54 @@ int SSH_write_X509(FILE *fp, X509 *x) {
   return ret;
 
 }
+
+ int get_ec_pubkey_from_bytes(int curve_name, uint8_t *point, size_t point_len, EVP_PKEY **pkey) {
+   int rc = 0;
+   EC_POINT *ecpoint = NULL;
+   EC_KEY *eckey = NULL;
+   EC_GROUP *group = EC_GROUP_new_by_curve_name(curve_name);
+   if(group == NULL)
+     return YKPIV_MEMORY_ERROR;
+   EC_GROUP_set_asn1_flag(group, OPENSSL_EC_NAMED_CURVE);
+   eckey = EC_KEY_new();
+   if(eckey == NULL) {
+     rc = YKPIV_MEMORY_ERROR;
+     goto create_ec_cleanup;
+   }
+   if(EC_KEY_set_group(eckey, group) <= 0) {
+     rc = YKPIV_GENERIC_ERROR;
+     goto create_ec_cleanup;
+   }
+   ecpoint = EC_POINT_new(group);
+   if(ecpoint == NULL) {
+     rc = YKPIV_MEMORY_ERROR;
+     goto create_ec_cleanup;
+   }
+   if(EC_POINT_oct2point(group, ecpoint, point, point_len, NULL) <= 0) {
+     rc = YKPIV_ARGUMENT_ERROR;
+     goto create_ec_cleanup;
+   }
+   if(EC_KEY_set_public_key(eckey, ecpoint) <= 0) {
+     rc = YKPIV_GENERIC_ERROR;
+     goto create_ec_cleanup;
+   }
+   *pkey = EVP_PKEY_new();
+   if(*pkey == NULL) {
+     rc = YKPIV_MEMORY_ERROR;
+     goto create_ec_cleanup;
+   }
+   if(EVP_PKEY_assign_EC_KEY(*pkey, eckey) <= 0) {
+     rc = YKPIV_GENERIC_ERROR;
+     goto create_ec_cleanup;
+   }
+
+   create_ec_cleanup:
+   EC_GROUP_clear_free(group);
+   if(ecpoint != NULL) {
+     EC_POINT_clear_free(ecpoint);
+   }
+   if(rc != YKPIV_OK && eckey != NULL) {
+     EC_KEY_free(eckey);
+   }
+   return rc;
+ }
\ No newline at end of file
