@@ -120,7 +120,7 @@ static void print_version(ykpiv_state *state, const char *output_file_name) {
 static bool sign_data(ykpiv_state *state, const unsigned char *in, size_t len, unsigned char *out,
     size_t *out_len, unsigned char algorithm, int key) {
 
-  unsigned char signinput[1024] = {0};
+  unsigned char signinput[YKPIV_OBJ_MAX_SIZE] = {0};
   if(YKPIV_IS_RSA(algorithm)) {
     size_t padlen = 0;
     switch (algorithm) {
@@ -176,7 +176,7 @@ static int yk_rsa_meth_sign(int dtype, const unsigned char *m, unsigned int m_le
   size_t yk_siglen = RSA_size(rsa);
   const RSA_METHOD *meth = RSA_get_method(rsa);
   const struct internal_key *key = RSA_meth_get0_app_data(meth);
-  unsigned char message[256] = {0};
+  unsigned char message[YKPIV_OBJ_MAX_SIZE] = {0};
 
   if(key->oid_len) {
     memcpy(message, key->oid, key->oid_len);
@@ -360,24 +360,8 @@ static bool generate_key(ykpiv_state *state, enum enum_slot slot,
         } else {
           nid = NID_secp384r1;
         }
-        eckey = EC_KEY_new();
-        group = EC_GROUP_new_by_curve_name(nid);
-        EC_GROUP_set_asn1_flag(group, OPENSSL_EC_NAMED_CURVE);
-        if (EC_KEY_set_group(eckey, group) != 1) {
-          fprintf(stderr, "Failed to set EC group.\n");
-          goto generate_out;
-        }
-        ecpoint = EC_POINT_new(group);
-
-        if (!EC_POINT_oct2point(group, ecpoint, point, point_len, NULL)) {
-          fprintf(stderr, "Failed to load public point.\n");
-          goto generate_out;
-        }
-        if (!EC_KEY_set_public_key(eckey, ecpoint)) {
-          fprintf(stderr, "Failed to set the public key.\n");
-          goto generate_out;
-        }
-        if (EVP_PKEY_set1_EC_KEY(public_key, eckey) != 1) {
+        res = get_ec_pubkey_from_bytes(nid, point, point_len, &public_key);
+        if(res != YKPIV_OK) {
           fprintf(stderr, "Failed to set EC public key.\n");
           goto generate_out;
         }
@@ -443,8 +427,36 @@ generate_out:
   return ret;
 }
 
-static bool reset(ykpiv_state *state) {
-  return ykpiv_util_reset(state) == YKPIV_OK;
+static bool reset(ykpiv_state *state, bool global) {
+  if (!global) {
+    if (ykpiv_util_reset(state) != YKPIV_OK) {
+      fprintf(stderr, "Reset failed, are pincodes blocked?\n");
+      return false;
+    }
+  } else {
+    fprintf(stderr,
+            "ALL data, including PIV data, in the YubiKey will be deleted. The action cannot be reversed!\n\nType 'y' to proceed: ");
+    char resp = fgetc(stdin);
+    if (resp == 'y') {
+      ykpiv_rc rc = ykpiv_global_reset(state);
+      if (rc != YKPIV_OK) {
+        if (rc == YKPIV_NOT_SUPPORTED) {
+          fprintf(stderr,
+                  "Global reset not supported on this YubiKey. Please refer to reset commands for specific applications instead\n");
+        } else if(rc == YKPIV_ARGUMENT_ERROR) {
+          fprintf(stderr, "Reset failed, is 'scp11' flag used?\n");
+        } else {
+          fprintf(stderr, "Reset failed\n");
+        }
+        return false;
+      }
+    } else {
+      fprintf(stderr, "Global reset operation aborted\n");
+      return true;
+    }
+  }
+  fprintf(stderr, "Successfully reset the application.\n");
+  return true;
 }
 
 static bool set_pin_retries(ykpiv_state *state, int pin_retries, int puk_retries, int verbose) {
@@ -515,13 +527,14 @@ static bool import_key(ykpiv_state *state, enum enum_key_format key_format,
 
     if(YKPIV_IS_RSA(algorithm)) {
       RSA *rsa_private_key = EVP_PKEY_get1_RSA(private_key);
-      unsigned char e[4] = {0};
+      unsigned char e[3] = {0};
       unsigned char p[256] = {0};
       unsigned char q[256] = {0};
       unsigned char dmp1[256] = {0};
       unsigned char dmq1[256] = {0};
       unsigned char iqmp[256] = {0};
       const BIGNUM *bn_e, *bn_p, *bn_q, *bn_dmp1, *bn_dmq1, *bn_iqmp;
+      int len_e, len_p, len_q, len_dmp1, len_dmq1, len_iqmp;
 
       int element_len = 0;
       switch(algorithm) {
@@ -545,43 +558,49 @@ static bool import_key(ykpiv_state *state, enum enum_key_format key_format,
       RSA_get0_key(rsa_private_key, NULL, &bn_e, NULL);
       RSA_get0_factors(rsa_private_key, &bn_p, &bn_q);
       RSA_get0_crt_params(rsa_private_key, &bn_dmp1, &bn_dmq1, &bn_iqmp);
-      if((set_component(e, bn_e, 3) == false) ||
+      len_e = sizeof(e);
+      if((set_component(e, bn_e, &len_e) == false) ||
          !(e[0] == 0x01 && e[1] == 0x00 && e[2] == 0x01)) {
         fprintf(stderr, "Invalid public exponent for import (only 0x10001 supported)\n");
         goto import_out;
       }
 
-      if(set_component(p, bn_p, element_len) == false) {
+      len_p = element_len;
+      if(set_component(p, bn_p, &len_p) == false) {
         fprintf(stderr, "Failed setting p component.\n");
         goto import_out;
       }
 
-      if(set_component(q, bn_q, element_len) == false) {
+      len_q = element_len;
+      if(set_component(q, bn_q, &len_q) == false) {
         fprintf(stderr, "Failed setting q component.\n");
         goto import_out;
       }
 
-      if(set_component(dmp1, bn_dmp1, element_len) == false) {
+      len_dmp1 = element_len;
+      if(set_component(dmp1, bn_dmp1, &len_dmp1) == false) {
         fprintf(stderr, "Failed setting dmp1 component.\n");
         goto import_out;
       }
 
-      if(set_component(dmq1, bn_dmq1, element_len) == false) {
+      len_dmq1 = element_len;
+      if(set_component(dmq1, bn_dmq1, &len_dmq1) == false) {
         fprintf(stderr, "Failed setting dmq1 component.\n");
         goto import_out;
       }
 
-      if(set_component(iqmp, bn_iqmp, element_len) == false) {
+      len_iqmp = element_len;
+      if(set_component(iqmp, bn_iqmp, &len_iqmp) == false) {
         fprintf(stderr, "Failed setting iqmp component.\n");
         goto import_out;
       }
 
       rc = ykpiv_import_private_key(state, key, algorithm,
-                                    p, element_len,
-                                    q, element_len,
-                                    dmp1, element_len,
-                                    dmq1, element_len,
-                                    iqmp, element_len,
+                                    p, len_p,
+                                    q, len_q,
+                                    dmp1, len_dmp1,
+                                    dmq1, len_dmq1,
+                                    iqmp, len_iqmp,
                                     NULL, 0,
                                     pp, tp);
     }
@@ -595,7 +614,7 @@ static bool import_key(ykpiv_state *state, enum enum_key_format key_format,
         element_len = 48;
       }
 
-      if(set_component(s_ptr, s, element_len) == false) {
+      if(set_component(s_ptr, s, &element_len) == false) {
         fprintf(stderr, "Failed setting ec private key.\n");
         goto import_out;
       }
@@ -1412,7 +1431,7 @@ static bool selfsign_certificate(ykpiv_state *state, enum enum_key_format key_fo
     int tbs_len = i2d_re_X509_tbs(x509, &tbs_data);
 
     // Sign the certificate data using the YubiKey
-    unsigned char yk_sig[64] = {0};
+    unsigned char yk_sig[512] = {0};
     size_t yk_siglen = sizeof(yk_sig);
     if (!sign_data(state, tbs_data, tbs_len, yk_sig, &yk_siglen, algorithm, key)) {
       fprintf(stderr, "Failed signing tbs certificate portion.\n");
@@ -1482,17 +1501,25 @@ selfsign_out:
   return ret;
 }
 
-static bool verify_pin(ykpiv_state *state, const char *pin) {
+static bool verify_pin(ykpiv_state *state, const char *pin, bool bio) {
   int tries = -1;
   ykpiv_rc res;
-  int len;
-  len = strlen(pin);
+  int len = 0;
 
-  if(len > 8) {
-    fprintf(stderr, "Maximum 8 digits of PIN supported.\n");
+  if (pin) {
+      len = strlen(pin);
+
+      if(len > 8) {
+        fprintf(stderr, "Maximum 8 digits of PIN supported.\n");
+      }
   }
 
-  res = ykpiv_verify(state, pin, &tries);
+  if (bio) {
+    fprintf(stderr, "Please scan your fingerprint on your YubiKey\n");
+    res = ykpiv_verify_bio(state, NULL, NULL, &tries, false);
+  } else {
+    res = ykpiv_verify(state, pin, &tries);
+  }
   if(res == YKPIV_OK) {
     return true;
   } else if(res == YKPIV_WRONG_PIN || res == YKPIV_PIN_LOCKED) {
@@ -2074,6 +2101,30 @@ static bool test_signature(ykpiv_state *state, enum enum_slot slot,
           }
         }
         break;
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+      case YKPIV_ALGO_ED25519:
+        {
+          EVP_MD_CTX *ctx;
+          int rc;
+          ctx = EVP_MD_CTX_new();
+          if (!ctx || EVP_DigestVerifyInit(ctx, NULL, NULL, NULL, pubkey) <= 0) {
+            fprintf(stderr, "Failed routine initialization\n");
+            EVP_MD_CTX_free(ctx); // It's OK if ctx is NULL
+            goto test_out;
+          }
+          rc = EVP_DigestVerify(ctx, signature, (int)sig_len, data, (int)data_len);
+          EVP_MD_CTX_free(ctx);
+          if(rc == 1) {
+            fprintf(stderr, "Successful EDDSA verification.\n");
+            ret = true;
+            goto test_out;
+          } else {
+            fprintf(stderr, "Failed EDDSA verification.\n");
+            goto test_out;
+          }
+        }
+        break;
+#endif
       default:
         fprintf(stderr, "Unknown algorithm.\n");
         goto test_out;
@@ -2139,7 +2190,7 @@ static bool test_decipher(ykpiv_state *state, enum enum_slot slot,
     if(YKPIV_IS_RSA(algorithm)) {
       unsigned char secret[32] = {0};
       unsigned char secret2[32] = {0};
-      unsigned char data[256] = {0};
+      unsigned char data[512] = {0};
       int len;
       size_t len2 = sizeof(data);
       RSA *rsa = EVP_PKEY_get1_RSA(pubkey);
@@ -2392,6 +2443,11 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "Warning, unable to reset locale\n");
   }
 
+  if (argc < 2) {
+    fprintf(stderr, "No actions detected. Use '--help' or '-h' for command manpage\n");
+    return EXIT_FAILURE;
+  }
+
   if(cmdline_parser(argc, argv, &args_info) != 0) {
     return EXIT_FAILURE;
   }
@@ -2456,6 +2512,7 @@ int main(int argc, char *argv[]) {
       case action_arg_changeMINUS_puk:
       case action_arg_unblockMINUS_pin:
       case action_arg_verifyMINUS_pin:
+      case action_arg_verifyMINUS_bio:
       case action_arg_setMINUS_mgmMINUS_key:
       case action_arg_setMINUS_chuid:
       case action_arg_setMINUS_ccc:
@@ -2482,7 +2539,7 @@ int main(int argc, char *argv[]) {
     return EXIT_FAILURE;
   }
 
-  if((rc = ykpiv_connect(state, args_info.reader_arg)) != YKPIV_OK) {
+  if((rc = ykpiv_connect_ex(state, args_info.reader_arg, args_info.scp11_given)) != YKPIV_OK) {
     fprintf(stderr, "Failed to connect to yubikey: %s.\n", ykpiv_strerror(rc));
     if (rc == YKPIV_PCSC_SERVICE_ERROR) {
       fprintf(stderr, "Try restarting the PCSC subsystem.\n");
@@ -2576,6 +2633,7 @@ int main(int argc, char *argv[]) {
       case action_arg_reset:
       case action_arg_requestMINUS_certificate:
       case action_arg_verifyMINUS_pin:
+      case action_arg_verifyMINUS_bio:
       case action_arg_changeMINUS_pin:
       case action_arg_changeMINUS_puk:
       case action_arg_unblockMINUS_pin:
@@ -2647,11 +2705,8 @@ int main(int argc, char *argv[]) {
         }
         break;
       case action_arg_reset:
-        if(reset(state) == false) {
-          fprintf(stderr, "Reset failed, are pincodes blocked?\n");
+        if(!reset(state, args_info.global_given)) {
           ret = EXIT_FAILURE;
-        } else {
-          fprintf(stderr, "Successfully reset the application.\n");
         }
         break;
       case action_arg_pinMINUS_retries:
@@ -2709,13 +2764,21 @@ int main(int argc, char *argv[]) {
           }
           pin = pinbuf;
         }
-        if(verify_pin(state, pin)) {
+        if(verify_pin(state, pin, false)) {
           fprintf(stderr, "Successfully verified PIN.\n");
         } else {
           ret = EXIT_FAILURE;
         }
         break;
       }
+      case action_arg_verifyMINUS_bio: {
+        if(verify_pin(state, NULL, true)) {
+          fprintf(stderr, "Successfully verified (fingerprint match).\n");
+        } else {
+            ret = EXIT_FAILURE;
+        }
+        break;
+      }
       case action_arg_changeMINUS_pin:
       case action_arg_changeMINUS_puk:
       case action_arg_unblockMINUS_pin: {
