@@ -164,58 +164,6 @@ gen_ec_key_cleanup:
   return rv;
 }
 
-CK_RV do_create_ec_key(CK_BYTE_PTR point, CK_ULONG point_len, int curve_name, ykcs11_pkey_t **pkey) {
-  CK_RV rv;
-  EC_POINT *ecpoint = NULL;
-  EC_KEY *eckey = NULL;
-  EC_GROUP *group = EC_GROUP_new_by_curve_name(curve_name);
-  if(group == NULL)
-    return CKR_HOST_MEMORY;
-  EC_GROUP_set_asn1_flag(group, OPENSSL_EC_NAMED_CURVE);
-  eckey = EC_KEY_new();
-  if(eckey == NULL) {
-    rv = CKR_HOST_MEMORY;
-    goto create_ec_cleanup;
-  }
-  if(EC_KEY_set_group(eckey, group) <= 0) {
-    rv = CKR_GENERAL_ERROR;
-    goto create_ec_cleanup;
-  }
-  ecpoint = EC_POINT_new(group);
-  if(ecpoint == NULL) {
-    rv = CKR_HOST_MEMORY;
-    goto create_ec_cleanup;
-  }
-  if(EC_POINT_oct2point(group, ecpoint, point, point_len, NULL) <= 0) {
-    rv = CKR_ARGUMENTS_BAD;
-    goto create_ec_cleanup;
-  }
-  if(EC_KEY_set_public_key(eckey, ecpoint) <= 0) {
-    rv = CKR_GENERAL_ERROR;
-    goto create_ec_cleanup;
-  }
-  EVP_PKEY_free(*pkey);
-  *pkey = EVP_PKEY_new();
-  if(*pkey == NULL) {
-    rv = CKR_HOST_MEMORY;
-    goto create_ec_cleanup;
-  }
-  if(EVP_PKEY_assign_EC_KEY(*pkey, eckey) <= 0) {
-    rv = CKR_GENERAL_ERROR;
-    goto create_ec_cleanup;
-  }
-  rv = CKR_OK;
-create_ec_cleanup:
-  EC_GROUP_clear_free(group);
-  if(ecpoint != NULL) {
-    EC_POINT_clear_free(ecpoint);
-  }
-  if(rv != CKR_OK && eckey != NULL) {
-    EC_KEY_free(eckey);
-  }
-  return rv;
-}
-
 CK_RV do_create_rsa_key(CK_BYTE_PTR mod, CK_ULONG mod_len, CK_BYTE_PTR exp, CK_ULONG exp_len, ykcs11_pkey_t **pkey) {
   CK_RV rv;
   RSA *rsa = NULL;
@@ -259,11 +207,9 @@ create_rsa_cleanup:
 }
 
 CK_RV do_create_public_key(CK_BYTE_PTR in, CK_ULONG in_len, CK_ULONG algorithm, ykcs11_pkey_t **pkey) {
-  int curve_name = get_curve_name(algorithm);
   CK_BYTE_PTR eob = in + in_len;
   unsigned long offs, len;
-
-  if (curve_name == 0) {
+  if (YKPIV_IS_RSA(algorithm)) {
     if(in >= eob)
       return CKR_GENERAL_ERROR;
 
@@ -293,8 +239,7 @@ CK_RV do_create_public_key(CK_BYTE_PTR in, CK_ULONG in_len, CK_ULONG algorithm,
 
     in += offs;    
     return do_create_rsa_key(mod, mod_len, in, len, pkey);
-  }
-  else {
+  } else {
     if(in >= eob)
       return CKR_GENERAL_ERROR;
 
@@ -306,14 +251,33 @@ CK_RV do_create_public_key(CK_BYTE_PTR in, CK_ULONG in_len, CK_ULONG algorithm,
       return CKR_GENERAL_ERROR;
 
     in += offs;
-    return do_create_ec_key(in, len, curve_name, pkey);
+
+    if (YKPIV_IS_EC(algorithm)) {
+      int curve_name = get_curve_name(algorithm);
+      return get_ec_pubkey_from_bytes(curve_name, in, len, pkey);
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+    } else if (YKPIV_IS_25519(algorithm)) {
+      if (algorithm == YKPIV_ALGO_ED25519) {
+        *pkey = EVP_PKEY_new_raw_public_key(EVP_PKEY_ED25519, NULL, in, len);
+      } else {
+        *pkey = EVP_PKEY_new_raw_public_key(EVP_PKEY_X25519, NULL, in, len);
+      }
+      if (*pkey == NULL) {
+        return CKR_HOST_MEMORY;
+      }
+      return CKR_OK;
+#endif
+    }
   }
+  DBG("Unsupported key algorithm");
+  return CKR_DATA_INVALID;
 }
 
 CK_RV do_sign_empty_cert(const char *cn, ykcs11_pkey_t *pubkey, ykcs11_pkey_t *pvtkey, ykcs11_x509_t **cert) {
   *cert = X509_new();
-  if (*cert == NULL)
+  if (*cert == NULL) {
     return CKR_HOST_MEMORY;
+  }
   X509_set_version(*cert, 2); // Version 3
   X509_NAME_add_entry_by_txt(X509_get_issuer_name(*cert), "CN", MBSTRING_ASC, (const unsigned char*)cn, -1, -1, 0);
   X509_NAME_add_entry_by_txt(X509_get_subject_name(*cert), "CN", MBSTRING_ASC, (const unsigned char*)cn, -1, -1, 0);
@@ -515,9 +479,14 @@ CK_KEY_TYPE do_get_key_type(ykcs11_pkey_t *key) {
     switch (EVP_PKEY_base_id(key)) {
     case EVP_PKEY_RSA:
       return CKK_RSA;
-
     case EVP_PKEY_EC:
       return CKK_EC;
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+    case EVP_PKEY_ED25519:
+      return CKK_EC_EDWARDS;
+    case EVP_PKEY_X25519:
+      return CKK_EC_MONTGOMERY;
+#endif
     }
   }
   return CKK_VENDOR_DEFINED; // Actually an error
@@ -538,6 +507,9 @@ CK_ULONG do_get_signature_size(ykcs11_pkey_t *key) {
     case EVP_PKEY_RSA:
       return EVP_PKEY_size(key);
     case EVP_PKEY_EC:
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+    case EVP_PKEY_ED25519:
+#endif
       switch(EVP_PKEY_bits(key)) {
       case 256:
         return 64;
@@ -571,6 +543,12 @@ CK_BYTE do_get_key_algorithm(ykcs11_pkey_t *key) {
       case 384:
         return YKPIV_ALGO_ECCP384;
       }
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+    case EVP_PKEY_ED25519:
+      return YKPIV_ALGO_ED25519;
+    case EVP_PKEY_X25519:
+      return YKPIV_ALGO_X25519;
+#endif
     }
   }
   return 0;
@@ -623,46 +601,54 @@ CK_RV do_get_public_exponent(ykcs11_pkey_t *key, CK_BYTE_PTR data, CK_ULONG_PTR
 }
 
 CK_RV do_get_public_key(ykcs11_pkey_t *key, CK_BYTE_PTR data, CK_ULONG_PTR len) {
-  const RSA *rsa = NULL;
-  unsigned char *p;
-
-  const EC_KEY *eck = NULL;
-  const EC_GROUP *ecg; // Alternative solution is to get i2d_PUBKEY and manually offset
-  const EC_POINT *ecp;
-  point_conversion_form_t pcf = POINT_CONVERSION_UNCOMPRESSED;
 
   switch(EVP_PKEY_base_id(key)) {
-  case EVP_PKEY_RSA:
+  case EVP_PKEY_RSA: {
+      const RSA *rsa = EVP_PKEY_get0_RSA(key);
 
-    rsa = EVP_PKEY_get0_RSA(key);
+      if (RSA_size(rsa) > *len) {
+        return CKR_BUFFER_TOO_SMALL;
+      }
 
-    if ((CK_ULONG)RSA_size(rsa) > *len) {
-      return CKR_BUFFER_TOO_SMALL;
+      CK_BYTE_PTR p = data;
+      if ((*len = i2d_RSAPublicKey(rsa, &p)) == 0) {
+        return CKR_FUNCTION_FAILED;
+      }
     }
-
-    p = data;
-
-    if ((*len = (CK_ULONG) i2d_RSAPublicKey(rsa, &p)) == 0) {
-      return CKR_FUNCTION_FAILED;
-    }
-
     break;
 
-  case EVP_PKEY_EC:
-    eck = EVP_PKEY_get0_EC_KEY(key);
-    ecg = EC_KEY_get0_group(eck);
-    ecp = EC_KEY_get0_public_key(eck);
+  case EVP_PKEY_EC: {
+      const EC_KEY *eck = EVP_PKEY_get0_EC_KEY(key);
+      const EC_GROUP *ecg = EC_KEY_get0_group(eck);
+      const EC_POINT *ecp = EC_KEY_get0_public_key(eck);
 
-    // Add the DER structure with length after extracting the point
-    data[0] = 0x04;
+      // Add the DER structure with length after extracting the point
+      data[0] = 0x04;
 
-    if ((*len = EC_POINT_point2oct(ecg, ecp, pcf, data + 2, *len - 2, NULL)) == 0) {
-      return CKR_FUNCTION_FAILED;
+      if ((*len = EC_POINT_point2oct(ecg, ecp, POINT_CONVERSION_UNCOMPRESSED, data + 2, *len - 2, NULL)) == 0) {
+        return CKR_FUNCTION_FAILED;
+      }
+
+      data[1] = *len;
+      *len += 2;
+    }
+    break;
+  case EVP_PKEY_ED25519:
+  case EVP_PKEY_X25519: {
+      size_t n = *len;
+      if(EVP_PKEY_get_raw_public_key(key, data, &n) != 1) {
+        return CKR_FUNCTION_FAILED;
+      }
+      ASN1_OCTET_STRING *a = ASN1_OCTET_STRING_new();
+      ASN1_OCTET_STRING_set(a, data, n);
+      if(i2d_ASN1_OCTET_STRING(a, NULL) > *len) {
+        ASN1_OCTET_STRING_free(a);
+        return CKR_BUFFER_TOO_SMALL;
+      }
+      CK_BYTE_PTR p = data;
+      *len = i2d_ASN1_OCTET_STRING(a, &p);
+      ASN1_OCTET_STRING_free(a);
     }
-
-    data[1] = *len;
-    *len += 2;
-
     break;
 
   default:
