@@ -71,7 +71,9 @@ static const token_mechanism token_mechanisms[] = {
   CKM_SHA_1, {0, 0, CKF_DIGEST},
   CKM_SHA256, {0, 0, CKF_DIGEST},
   CKM_SHA384, {0, 0, CKF_DIGEST},
-  CKM_SHA512, {0, 0, CKF_DIGEST}
+  CKM_SHA512, {0, 0, CKF_DIGEST},
+  CKM_EC_EDWARDS_KEY_PAIR_GEN, {255, 255, CKF_HW | CKF_GENERATE_KEY_PAIR | CKF_EC_F_P | CKF_EC_NAMEDCURVE | CKF_EC_UNCOMPRESS},
+  CKM_EC_MONTGOMERY_KEY_PAIR_GEN, {255, 255, CKF_HW | CKF_GENERATE_KEY_PAIR | CKF_EC_F_P | CKF_EC_NAMEDCURVE | CKF_EC_UNCOMPRESS}
 };
 
 // The commented out objects below are either not supported (PIV_DATA_OBJ_BITGT) or requires authentication.
@@ -299,14 +301,16 @@ CK_RV token_change_pin(ykpiv_state *state, CK_USER_TYPE user_type, CK_UTF8CHAR_P
 CK_RV token_login(ykpiv_state *state, CK_USER_TYPE user, CK_UTF8CHAR_PTR pin, CK_ULONG pin_len) {
 
   ykpiv_rc res;
+  int tries = 0;
 
-  if (pin_len >= YKPIV_MIN_PIN_LEN && pin_len <= YKPIV_MAX_PIN_LEN) {
+  if (pin_len == 0) {
+    res = ykpiv_verify_bio(state, NULL, NULL, &tries, false);
+  } else if (pin_len >= YKPIV_MIN_PIN_LEN && pin_len <= YKPIV_MAX_PIN_LEN) {
     char term_pin[YKPIV_MAX_PIN_LEN + 1] = {0};
 
     memcpy(term_pin, pin, pin_len);
     term_pin[pin_len] = 0;
 
-    int tries = 0;
     res = ykpiv_verify(state, term_pin, &tries);
 
     OPENSSL_cleanse(term_pin, pin_len);
@@ -396,6 +400,14 @@ CK_RV token_generate_key(ykpiv_state *state, gen_info_t *gen, CK_BYTE key, CK_BY
     case YKPIV_ALGO_ECCP384:
       break;
 
+    case YKPIV_ALGO_ED25519:
+    case YKPIV_ALGO_X25519:
+      if (!is_version_compatible(state, 5, 7, 0)) {
+        DBG("ED25519 and X25519 key types are only available with YubiKeys with version number 5.7.0 or later");
+        return CKR_FUNCTION_NOT_SUPPORTED;
+      }
+      break;
+
     default:
       return CKR_FUNCTION_FAILED;
   }
@@ -415,6 +427,8 @@ CK_RV token_generate_key(ykpiv_state *state, gen_info_t *gen, CK_BYTE key, CK_BY
     case YKPIV_PINPOLICY_ALWAYS:
     case YKPIV_PINPOLICY_ONCE:
     case YKPIV_PINPOLICY_NEVER:
+    case YKPIV_PINPOLICY_MATCH_ONCE:
+    case YKPIV_PINPOLICY_MATCH_ALWAYS:
         break;
     default: 
         return CKR_FUNCTION_FAILED;
@@ -445,7 +459,7 @@ CK_RV token_generate_key(ykpiv_state *state, gen_info_t *gen, CK_BYTE key, CK_BY
   if((res = ykpiv_transfer_data(state, templ, in_data, in_ptr - in_data, data, &recv_len, &sw)) != YKPIV_OK) {
     return yrc_to_rv(res);
   }
-  if((res = ykpiv_translate_sw(sw)) != YKPIV_OK) {
+  if((res = ykpiv_translate_sw_ex(__FUNCTION__, sw)) != YKPIV_OK) {
     return yrc_to_rv(res);
   }
 
